import { Component, h } from 'preact';

/** Creates a new store, which is a tiny evented state container.
 *  @example
 *    let store = createStore();
 *    store.subscribe( state => console.log(state) );
 *    store.setState({ a: 'b' });   // logs { a: 'b' }
 *    store.setState({ c: 'd' });   // logs { c: 'd' }
 */
function createStore(state) {
	if ( state === void 0 ) state={};

	var listeners = [];

	return {
		setState: function setState(update) {
			state = assign({}, state, update);
			listeners.forEach( function (f) { f(state); });
		},
		subscribe: function subscribe(f) {
			listeners.push(f);
		},
		unsubscribe: function unsubscribe(f) {
			var i = listeners.indexOf(f);
			listeners.splice(i, !!~i);
		},
		getState: function getState() {
			return state;
		}
	};
}


/** Provides its props into the tree as context.
 *  @example
 *    let store = createStore();
 *    <Provider store={store}><App /></Provider>
 */
var Provider = (function (Component$$1) {
	function Provider () {
		Component$$1.apply(this, arguments);
	}

	if ( Component$$1 ) Provider.__proto__ = Component$$1;
	Provider.prototype = Object.create( Component$$1 && Component$$1.prototype );
	Provider.prototype.constructor = Provider;

	Provider.prototype.getChildContext = function getChildContext () {
		var context = assign({}, this.props);
		delete context.children;
		return context;
	};
	Provider.prototype.render = function render (ref) {
		var children = ref.children;

		return children[0];
	};

	return Provider;
}(Component));


/** Wire a component up to the store. Passes state as props, re-renders on change.
 *  @param {Function|Array|String} mapStateToProps  A function (or any `select()` argument) mapping of store state to prop values.
 *  @example
 *    const Foo = connect('foo,bar')( ({ foo, bar }) => <div /> )
 *  @example
 *    @connect( state => ({ foo: state.foo, bar: state.bar }) )
 *    export class Foo { render({ foo, bar }) { } }
 */
function connect(mapToProps, actions) {
	if (typeof mapToProps!=='function') { mapToProps = select(mapToProps); }
	return function (Child) { return (
		(function (Component$$1) {
			function Wrapper(props, ref) {
				var this$1 = this;
				var store = ref.store;

				Component$$1.call(this);
				this.state = mapToProps(store ? store.getState() : {}, props);
				this.actions = actions ? actions(store) : { store: store };
				this.update = function () {
					var mapped = mapToProps(store ? store.getState() : {}, this$1.props);
					if (!shallowEqual(mapped, this$1.state)) {
						this$1.setState(mapped);
					}
				};
			}

			if ( Component$$1 ) Wrapper.__proto__ = Component$$1;
			Wrapper.prototype = Object.create( Component$$1 && Component$$1.prototype );
			Wrapper.prototype.constructor = Wrapper;
			Wrapper.prototype.componentDidMount = function componentDidMount () {
				this.context.store.subscribe(this.update);
			};
			Wrapper.prototype.componentWillUnmount = function componentWillUnmount () {
				this.context.store.unsubscribe(this.update);
			};
			Wrapper.prototype.render = function render (props, state) {
				return h( Child, assign({}, this.actions, props, state));
			};

			return Wrapper;
		}(Component))
	); };
}


/** select('foo,bar') creates a function of the form: ({ foo, bar }) => ({ foo, bar }) */
function select(properties) {
	if (typeof properties==='string') { properties = properties.split(','); }
	return function (state) {
		var selected = {};
		for (var i=0; i<properties.length; i++) {
			selected[properties[i]] = state[properties[i]];
		}
		return selected;
	};
}


// eslint-disable-next-line
function assign(obj) {
	var arguments$1 = arguments;

	for (var i=1; i<arguments.length; i++) {
		var props = arguments$1[i];
		for (var j in props) { obj[j] = props[j]; }
	}
	return obj;
}


/** Returns a boolean indicating if all keys and values match between two objects. */
function shallowEqual(a, b) {
	for (var i in a) { if (a[i]!==b[i]) { return false; } }
	for (var i$1 in b) { if (!(i$1 in a)) { return false; } }
	return true;
}

export { createStore, Provider, connect, select };
//# sourceMappingURL=unistore.es.js.map
